@@ -1270,16 +1270,64 @@ void View::createMenus()
 	rootMenu->adjust();
 }
 
+/** Handle click on either create new set or edit existing set.
+ * Run editor and create test game if single level is to be tested.
+ * Since editor would need the view to run the game (not sure if a
+ * second view would be ok because of the shared SDL window?), we
+ * shortly leave the editor and run the test game from inside the view.
+ * Messy solution, I know, but that's just how we roll...
+ */
+void View::handleEditor(int type)
+{
+	string fname;
+
+	/* get file name from menu or select dialog */
+	if (type == AID_EDITNEWSET)
+		fname = config.edit_setname;
+	else {
+		darkenScreen();
+		selectDlg.init(SDT_CUSTOMONLY);
+		if (selectDlg.run())
+			fname = selectDlg.get();
+		else {
+			if (selectDlg.quitRcvd())
+				quitReceived = true;
+			return;
+		}
+	}
+
+	/* run editor */
+	while (1) {
+		editor.run(fname);
+		if (editor.quitRcvd()) {
+			quitReceived = true;
+			return;
+		}
+		/* test level? */
+		if (editor.testRequested()) {
+			if (cgame.initTestlevel(editor.getCurrentLevel()->title,
+					editor.getCurrentLevel()->author,
+					editor.getCurrentLevel()->bricks,
+					editor.getCurrentLevel()->extras) == 0) {
+				dim();
+				run();
+			}
+		} else
+			break; /* editor normally exited */
+	}
+}
+
 void View::runMenu()
 {
 	SDL_Event ev;
 	Ticks ticks;
 	MenuItemSub *subItem;
 	MenuItemBack *backItem;
-	bool changingKey = false, newEvent = false;
+	bool changingKey = false, newEvent = false, changedKey = false;
 	int aid = AID_NONE;
 
 	curMenu = rootMenu.get();
+	curMenu->resetSelection();
 	renderMenu();
 
 	while (!quitReceived) {
@@ -1287,6 +1335,7 @@ void View::runMenu()
 		newEvent = false;
 		if (SDL_PollEvent(&ev)) {
 			newEvent = true;
+			changedKey = false;
 			if (ev.type == SDL_QUIT)
 				quitReceived = true;
 			if (changingKey && ev.type == SDL_KEYDOWN) {
@@ -1302,10 +1351,12 @@ void View::runMenu()
 				default:
 					keyItem->setKey(ev.key.keysym.scancode);
 					changingKey = false;
+					changedKey = true;
 					break;
 				}
 			} else if (ev.type == SDL_KEYDOWN &&
-				ev.key.keysym.scancode == SDL_SCANCODE_ESCAPE) {
+					(ev.key.keysym.scancode == SDL_SCANCODE_ESCAPE ||
+					ev.key.keysym.scancode == SDL_SCANCODE_BACKSPACE)) {
 				Menu *lm = curMenu->getLastMenu();
 				if (lm)
 					curMenu = lm;
@@ -1317,7 +1368,8 @@ void View::runMenu()
 		curMenu->update(ticks.get());
 
 		/* handle events in items */
-		if (newEvent && !changingKey && (aid = curMenu->handleEvent(ev))) {
+		if (newEvent && !changingKey && !changedKey &&
+					(aid = curMenu->handleEvent(ev))) {
 			if (aid != AID_FOCUSCHANGED)
 				mixer.play(theme.sMenuClick);
 			switch (aid) {
@@ -1331,6 +1383,7 @@ void View::runMenu()
 				subItem = dynamic_cast<MenuItemSub*>(curMenu->getCurItem());
 				if (subItem) { /* should never fail */
 					curMenu = subItem->getSubMenu();
+					curMenu->resetSelection();
 				} else
 					_logerr("Oops, submenu not found...\n");
 				break;
@@ -1338,6 +1391,7 @@ void View::runMenu()
 				backItem = dynamic_cast<MenuItemBack*>(curMenu->getCurItem());
 				if (backItem) { /* should never fail */
 					curMenu = backItem->getLastMenu();
+					curMenu->resetSelection();
 				} else
 					_logerr("Oops, last menu not found...\n");
 				break;
@@ -1365,6 +1419,7 @@ void View::runMenu()
 				init(themeNames[config.theme_id],
 					MainWindow::getModeResolution(config.mode));
 				curMenu = graphicsMenu;
+				curMenu->resetSelection();
 				break;
 			case AID_RESUME:
 				if (resumeGame()) {
@@ -1402,24 +1457,11 @@ void View::runMenu()
 				ticks.reset();
 				break;
 			case AID_EDITNEWSET:
-				editor.run(config.edit_setname);
-				if (editor.quitRcvd())
-					quitReceived = true;
+			case AID_EDITCUSTOM:
+				handleEditor(aid);
 				dim();
 				ticks.reset();
 				break;
-			case AID_EDITCUSTOM:
-				darkenScreen();
-				selectDlg.init(SDT_CUSTOMONLY);
-				if (selectDlg.run()) {
-					editor.run(selectDlg.get());
-					if (editor.quitRcvd())
-						quitReceived = true;
-					dim();
-					ticks.reset();
-				} else if (selectDlg.quitRcvd())
-					quitReceived = true;
-				break;
 			}
 		}
 
